{-----------------------------------------------------------------------------

 ChiFu : a tetris-like game

-----------------------------------------------------------------------------}
program ChiFu;

uses
  CF_KIO,
  CF_Scor,
  CF_Grph,
  CF_Play,
  CF_File,
  CF_Pict,
  CF_Snd,
  CF_Set,
  CF_Time;

var
  Finished     : boolean;
  Count        : byte;
  UserResponse : char;
  Option       : TOption;


{ Shows the current settings }
procedure ShowSettings;

var
  Count   : byte;
  StrText : string;
  Setting : TSetting;

begin
  with Screen do begin
    { Show the sound status }
    CursorPos := OptionPos [SoundOnOff];
    if SoundOn then begin
      ShowText (LeftArrow + ' Sound is On  ' + RightArrow);
    end { then }
    else begin
      ShowText (LeftArrow + ' Sound is Off ' + RightArrow);
    end; { else }

    { The amount of full rows }
    CursorPos := OptionPos [ChangeFullRows];
    Str (FullRows, StrText);
    StrText := LeftArrow + ' Start with ' + StrText + ' Full Row';
    if FullRows <> 1 then begin
      StrText := StrText + 's';
    end { then }
    else begin
      StrText := ' ' + StrText;
    end; { else }
    ShowText (StrText + ' ' + RightArrow + ' ');

    { And the chosen play mode }
    CursorPos := OptionPos [ChangePlayMode];
    StrText := LeftArrow + ' Play in ' + PlayModeNames [PlayMode] + ' Mode ';
    if PlayMode <> Normal then begin
      StrText := StrText + ' ';
    end; { then }
    ShowText (StrText + RightArrow);
  end; { with }
end; { ShowSettings }

{ Writes the main menu to screen }
procedure WriteMainMenu;

begin
  { Show the settings }
  ShowSettings;

  { And all static things }
  with Screen do begin
    { Show the menu options }
    CursorPos := OptionPos [PlayGame];
    ShowText ('Play New Game');

    CursorPos := OptionPos [ShowHi];
    ShowText ('Show Hall of Fame');

    CursorPos := OptionPos [ExitGame];
    ShowText ('Exit Game');

    CursorPos := OptionPos [Instructions];
    ShowText ('Game Instructions');

    { And special instructions }
    CursorPos := TitlePos;
    ShowText ('Chi Fu @ by Carl Colijn');

    CursorPos := Bottom;
    ShowText ('(Esc) during game will abort game.');
    ScrollUp;
    ScrollUp;
    ShowText ('Any other key will unlock the pause mode again.');
    ScrollUp;
    ScrollUp;
    ShowText ('When playing, type (P) to pause the game.');
  end; { with }
end; { WriteMainMenu }


{ Shows the current option or settings box }
procedure ShowCurrentBox;

begin
  Screen.ShowOptionBox (Option);
end; { ShowCurrentBox }

{ Counts visibily down from 5 to 0 }
procedure CountDown;

var
  TextLine,
  NumberText : string;
  CountNr    : byte;

begin
  { Set the clock to 1 second delay }
  Clock.TickTime := 100;

  { Show instructions }
  Screen.CursorPos := Screen.Bottom;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ShowText ('Get ready;');
  Screen.ScrollDown;
  Screen.ScrollDown;

  { And start countdown }
  Clock.Reset;
  for CountNr := 3 downto 1 do begin
    repeat until Clock.ClockTicked;
    Str (CountNr, NumberText);
    TextLine := 'Game starts in T minus ' + NumberText + ' second';
    Speaker.PlayTune;
    if CountNr <> 1 then begin
      TextLine := TextLine + 's';
    end; { then }
    Screen.ShowText (TextLine + ' . . . ');
  end; { for }
  repeat until Clock.ClockTicked;

  { Countdown is over! }
  Screen.ShowText ('GAME ON!                              ');
  Speaker.PlayTune;

  { And wait 1 more second }
  Clock.Reset;
  repeat until Clock.ClockTicked;
  Screen.ShowText ('        ');
  Screen.ScrollUp;
  Screen.ScrollUp;
  Screen.ShowText ('          ');
end; { CountDown }

{ Plays one game }
procedure PlayOneGame;

var
  CycleNr,
  AmBlockDropped,
  AmStonesDeleted : word;
  RoundFinished,
  PlayerDead,
  Finished        : boolean;
  UserResponse    : char;

begin
  { Initialize the field }
  Field.ClearField;
  Field.MakeFullRows;
  Screen.DrawField;
  Screen.DeleteOptionBox (Option);

  { Let the user get ready for the game }
  CountDown;

  { And the score }
  Scores [PlayMode].Score := 0;

  { And the clock }
  Clock.TickTime := 50;  { 1/2 sec }
  Clock.WaitTime := 25;  { 1/4 sec }

  { And play a game }
  AmBlockDropped := 0;
  PlayerDead := false;
  repeat
    { Initialize the first block }
    Block.MakeNewBlock;
    PlayerDead := (Block.Stuck);

    Screen.DrawBlock;

    { Show the score so far }
    Scores [PlayMode].ShowScore;

    RoundFinished := false;
    Clock.Reset;
    while not RoundFinished and not PlayerDead do begin
      { Look if the user is interacting }
      if KeyBoard.KeyPressed then begin
        { Yes -> look what interaction is going on }
        UserResponse := KeyBoard.GetKey;

        if UserResponse = Up then begin
          Block.TurnUp;
          Screen.DrawBlock;
        end { then }
        else if UserResponse = Down then begin
          Block.TurnDown;
          Screen.DrawBlock;
        end { else then }
        else if UserResponse = Left then begin
          Screen.DeleteBlock;
          Block.MoveLeft;
          Screen.DrawBlock;
        end { else then }
        else if UserResponse = Right then begin
          Screen.DeleteBlock;
          Block.MoveRight;
          Screen.DrawBlock;
        end { else then }
        else if UserResponse = ' ' then begin
          repeat
            Screen.DeleteBlock;
            Block.MoveDown;
            Screen.DrawBlock;
          until Block.Stuck;
          RoundFinished := true;
        end { else then }
        else if UserResponse = 'p' then begin
          Screen.ShowPauseScreen;
          WriteMainMenu;
          Screen.DrawBorder;
          Screen.DrawField;
          Screen.DrawBlock;
          Clock.Reset;
          Scores [PlayMode].ShowScore;
        end { else then }
        else if UserResponse = Esc then begin
          ShowCurrentBox;
          Screen.DeleteBlock;
          Exit;
        end; { else then }
      end; { then }

      { Move the block one down, if the time is right }
      if Clock.ClockTicked and not RoundFinished then begin
        Screen.DeleteBlock;
        Block.MoveDown;
        Screen.DrawBlock;
        if Block.Stuck then begin
          RoundFinished := true;
        end; { then }
      end; { then }
    end; { while }

    { Put the block in the field }
    Field.InsertBlock;

    { And adjust the playing speed }
    Inc (AmBlockDropped);
    if AmBlockDropped <= 400 then begin
      Clock.TickTime := 50 - AmBlockDropped div 8;
      Clock.WaitTime := 25 - AmBlockDropped div 16;
    end; { then }

    { Check on full rows (increase score) }
    if not PlayerDead then begin
      { Get rid of possible stones }
      CycleNr := 1;
      repeat
        AmStonesDeleted := Field.CheckRows;
        Scores [PlayMode].Score := Scores [PlayMode].Score + AmStonesDeleted * CycleNr;
        Inc (CycleNr);

        if AmStonesDeleted <> 0 then begin
          { Have a brief pause }
          Clock.Wait;

          { Play a tune }
          Speaker.PlayTune;

          { Show the new field }
          Screen.DrawField;
        end; { then }
      until AmStonesDeleted = 0;
    end; { then }
  until PlayerDead;

  { And if the score fits in the list, }
  if Scores [PlayMode].IsHiscore then begin
    { Add name to list, if possible }
    Scores [PlayMode].AddScore;
    Screen.ClearScreen;
    Screen.DrawBorder;
    Screen.DrawField;
    WriteMainMenu;
    ScoreFile.WriteScores;
  end; { then }
  ShowCurrentBox;
end; { PlayOneGame }

{ Shows game instructions to screen }
procedure ShowInstructions;

begin
  with Screen do begin
    { Prepare the screen }
    ClearScreen;
    CursorPos := Home;
    ScrollDown;
    ShowText ('              Chi Fu Game Instructions');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('     Warning : the game you are about to play is');
    ScrollDown;
    ShowText ('      highly addictive. Use it at your own risk.');
    ScrollDown;
    ScrollDown;
    ShowText ('The purpose of the game is quite similar to all other');
    ScrollDown;
    ShowText ('  tetris-like games; some blocks will fall and you');
    ScrollDown;
    ShowText ('  have to arrange them in such a way that filled up');
    ScrollDown;
    ShowText ('rows can be exterminated. But where all other tetris-');
    ScrollDown;
    ShowText ('   look-alikes have a multiple amount of different');
    ScrollDown;
    ShowText (' blocks that have to be rotated, this game has only');
    ScrollDown;
    ShowText ('   one type of block that cannot even be rotated.');
    ScrollDown;
    ScrollDown;
    ShowText ('This would get quite boring in a while, if it wasn''t');
    ScrollDown;
    ShowText ('  for the fact that this simplicity is compensated');
    ScrollDown;
    ShowText ('  for by six types of different stones. Each block');
    ScrollDown;
    ShowText ('   is made up of three of these, in random order.');
    ScrollDown;
    ShowText ('  Purpose now is to create rows of similar types of');
    ScrollDown;
    ShowText ('  stones. These rows might be horizontal, vertical');
    ScrollDown;
    ShowText ('   or diagonal; as long as they are at least three');
    ScrollDown;
    ShowText ('        stones long, they will be taken out.');
    ScrollDown;
    ScrollDown;
    ShowText ('To make the game a little more playable, you are (via');
    ScrollDown;
    ShowText ('   the up & down keys) also allowed to rotate the');
    ScrollDown;
    ShowText (' stones through the block. In this way, you have to');
    ScrollDown;
    ShowText ('  place your stones in a cunning way. Because after');
    ScrollDown;
    ShowText ('rows are being taken out, all other stones will fall');
    ScrollDown;
    ShowText ('  down and the field is again checked on full rows.');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('  Press any key to show the next instruction screen.');

    Keyboard.WaitKeyPress;
    Keyboard.GetKey;
    ClearScreen;
    CursorPos := Home;

    ShowText ('              Chi Fu Game Instructions');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText (' There is only one goal in this game; this being to');
    ScrollDown;
    ShowText (' get as much points as possible. You will get points');
    ScrollDown;
    ShowText ('for every stone that is being deleted from existance.');
    ScrollDown;
    ScrollDown;
    ShowText ('Once a block has been dropped, the program will check');
    ScrollDown;
    ShowText ('   on full rows. Every stone that is being deleted');
    ScrollDown;
    ShowText (' from existance will bring you one extra point. But');
    ScrollDown;
    ShowText ('after this deletion, the remaining stones on top will');
    ScrollDown;
    ShowText ('  fall down and there will be another check on full');
    ScrollDown;
    ShowText ('rows. These stones then will bring up two points, and');
    ScrollDown;
    ShowText (' so on. Therefore, the higher the cycle, the higher');
    ScrollDown;
    ShowText ('      the points that the stones will give you.');
    ScrollDown;
    ScrollDown;
    ShowText (' This might be a good reason to ''save up stones'' by');
    ScrollDown;
    ShowText ('   not deleting them immediately, but to keep them');
    ScrollDown;
    ShowText (' somewhere close together with some other stones in');
    ScrollDown;
    ShowText ('  between. Removal of these stones then results in');
    ScrollDown;
    ShowText ('          higher points for the other ones.');
    ScrollDown;
    ScrollDown;
    ShowText ('Therefore it might (at some points) be more clever to');
    ScrollDown;
    ShowText ('  think ahead and place a new block in a strategic');
    ScrollDown;
    ShowText (' place, rather than getting rid of some easy stones.');
    ScrollDown;
    ScrollDown;
    ShowText ('  Now you might think: "but this is so easy, even I');
    ScrollDown;
    ShowText ('could do that and end up high in the hiscore list! "');
    ScrollDown;
    ShowText ('  But I can assure you that this is a mistake: the');
    ScrollDown;
    ShowText (' further the game progresses, the faster it will go.');
    ScrollDown;
    ScrollDown;
    ShowText (' This speeding-up is controlled purely by the amount');
    ScrollDown;
    ShowText ('  of stones you have already dropped into the pit,');
    ScrollDown;
    ShowText ('     so be carefull to get as much points out of');
    ScrollDown;
    ShowText ('               every stone as possible.');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('  Press any key to show the next instruction screen.');

    Keyboard.WaitKeyPress;
    Keyboard.GetKey;
    ClearScreen;
    CursorPos := Home;

    ShowText ('              Chi Fu Game Instructions');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText (' The game can be played in three modes, being Small,');
    ScrollDown;
    ShowText ('   Normal and Large mode. Each of these modes has');
    ScrollDown;
    ShowText ('   other settings for a few game characteristics.');
    ScrollDown;
    ShowText ('  Small mode for instance uses a smaller amount of');
    ScrollDown;
    ShowText (' different stones than the other two, but the field');
    ScrollDown;
    ShowText ('            itself is also much smaller.');
    ScrollDown;
    ScrollDown;
    ShowText ('This compensates each other quite well; this enables');
    ScrollDown;
    ShowText ('  you to get the same overall score for each game.');
    ScrollDown;
    ShowText (' The hiscores are being separated between the modes');
    ScrollDown;
    ShowText ('  however; each mode has it''s own hi-score table.');
    ScrollDown;
    ScrollDown;
    ShowText ('Another option you are able to change in the game is');
    ScrollDown;
    ShowText (' the amount of full rows you will start with. These');
    ScrollDown;
    ShowText ('  rows will then be filled when you start the game.');
    ScrollDown;
    ShowText (' But of course these rows do not contain any ''full''');
    ScrollDown;
    ShowText ('   rows, so the only effect this option has is to');
    ScrollDown;
    ShowText ('           make things harder on yourself.');
    ScrollDown;
    ScrollDown;
    ShowText (' One positive thing however about starting with some');
    ScrollDown;
    ShowText (' full rows: since the game speeding-up is controlled');
    ScrollDown;
    ShowText (' solely by the amount of stones you havedropped into');
    ScrollDown;
    ShowText ('  the pit, removing of these extra stones will not');
    ScrollDown;
    ShowText ('result in any extra speeding up. This will allow you');
    ScrollDown;
    ShowText ('          to get higher scores in the end.');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('       Press any key to see the game controls.');

    Keyboard.WaitKeyPress;
    Keyboard.GetKey;
    ClearScreen;
    CursorPos := Home;

    ShowText ('               Chi Fu Game Controls');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('                UP/DOWN cursor keys');
    ScrollDown;
    ScrollDown;
    ShowText ('  will cycle the colors up & down through the block');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('               LEFT/RIGHT cursor keys');
    ScrollDown;
    ScrollDown;
    ShowText ('will move the block left and right through the field');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('                        SPACE');
    ScrollDown;
    ScrollDown;
    ShowText ('      immediately drops the block to the bottom');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('                          P');
    ScrollDown;
    ScrollDown;
    ShowText ('             puts the game in pause mode');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('                       ESCAPE');
    ScrollDown;
    ScrollDown;
    ShowText ('      when playing a game terminates the game;');
    ScrollDown;
    ShowText ('          when in main menu exits the game');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText (' Chi Fu @ was brought to you in 1996 by Carl Colijn.');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('      Press any key to see the acknowledgements.');

    Keyboard.WaitKeyPress;
    Keyboard.GetKey;
    ClearScreen;
    CursorPos := Home;

    ShowText ('            Chi Fu Game Acknowledgements');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('Many Thanx to the following people, without whom the');
    ScrollDown;
    ShowText ('   testing and clarifying of this game would have');
    ScrollDown;
    ShowText ('            been totally on my own hands:');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('Tineke for testing the game and just being interested');
    ScrollDown;
    ShowText ('          in the programming side of it all');
    ScrollDown;
    ShowText ('       (finally somebody who understands me!);');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('  Edwin for also testing this game and giving quite');
    ScrollDown;
    ShowText ('  useable hints as to how to improve the prototype;');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('    Jan Jelle for helping me clarifying the game');
    ScrollDown;
    ShowText ('   instructions so that everybody would be able');
    ScrollDown;
    ShowText ('                 to understand them;');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText (' And Nathalie for just playing this game (too) many');
    ScrollDown;
    ShowText (' times and telling me everything was going O.K. the');
    ScrollDown;
    ShowText (' way it was (sorry to have made you an addict. . .)');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('      Press any key to see the author''s page.');

    Keyboard.WaitKeyPress;
    Keyboard.GetKey;
    ClearScreen;
    CursorPos := Home;

    ShowText ('               Chi Fu Author''s Page');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('This game was developed by Carl Colijn. Orginal idea');
    ScrollDown;
    ShowText ('came from a very addictive game for the Commodore 64');
    ScrollDown;
    ShowText ('(yes, I had one too!), which this game is based upon.');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText (' This game is still under development. Things that I');
    ScrollDown;
    ShowText ('          wish to enhance and/or alter are:');
    ScrollDown;
    ShowText ('                  -More play modes;');
    ScrollDown;
    ShowText ('          -Better looking Hi-Score screen;');
    ScrollDown;
    ShowText ('                 -Two-player mode??!');
    ScrollDown;
    ShowText (' Anyway, all this will just have to wait for later.');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('  This game is intended to be completely ShareWare,');
    ScrollDown;
    ShowText ('which (in my opinion) means that all copying must be');
    ScrollDown;
    ShowText ('free of charge and must also be encouraged. However,');
    ScrollDown;
    ShowText ('when distributing this program, please pay notice to');
    ScrollDown;
    ShowText (' also include all files accompanying this game; they');
    ScrollDown;
    ShowText ('         are listed in the README.TXT file.');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('  Finally, for all you out there who are willing to');
    ScrollDown;
    ShowText ('give POSITIVE critisism, who want to report bugs (not');
    ScrollDown;
    ShowText ('  the one''s walking in your garden, please...), or');
    ScrollDown;
    ShowText ('just want to show financial support (''encouragement');
    ScrollDown;
    ShowText ('             fee''), my mail addres is:');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('                     Carl Colijn');
    ScrollDown;
    ShowText ('                   Boomkwekerij 1');
    ScrollDown;
    ShowText ('                       2635 KC');
    ScrollDown;
    ShowText ('                      Den Hoorn');
    ScrollDown;
    ShowText ('                   The Netherlands');
    ScrollDown;
    ScrollDown;
    ScrollDown;
    ShowText ('      Press any key to return to the main menu.');
{    ShowText ('0123456789abcdefghijklmnopqponmlkjihgfedcba9876543210');
 }
    Keyboard.WaitKeyPress;
    Keyboard.GetKey;
    ClearScreen;

    DrawBorder;
    DrawField;
    WriteMainMenu;
    ShowCurrentBox;
  end; { with }
end; { ShowInstructions }



begin
  { Standard menu cursor position }
  Option := PlayGame;

  { And keep showing that nice title screen for a while }
  Clock.TickTime := 700;
  repeat until Clock.ClockTicked;

  { Set the screen }
  Screen.ClearScreen;

  { Show an empty field }
  Field.ClearField;
  Screen.DrawBorder;
  Screen.DrawField;

  { And write the main menu }
  WriteMainMenu;
  ShowCurrentBox;

  { Not finished yet }
  Finished := false;
  repeat
    { Get the user's response }
    KeyBoard.CaseOn := false;
    Keyboard.EmptyBuffer;
    UserResponse := KeyBoard.GetSelectedKey (Up + Down + Left + Right +
                                             ' ' + Enter + Esc);

    { And act according to response }
    if (UserResponse = ' ') or (UserResponse = Enter) then begin
      { Option chosen }
      case Option of
        PlayGame : PlayOneGame;
        ShowHi   : begin
                     Scores [PlayMode].ShowHiScores;

                     Screen.ShowText ('Press any key to return to main menu . . .');
                     Keyboard.WaitKeyPress;
                     Keyboard.GetKey;

                     Screen.ClearScreen;
                     Screen.DrawBorder;
                     Screen.DrawField;
                     WriteMainMenu;
                     ShowCurrentBox;
                   end;
        ExitGame : Finished := true;
        Instructions : ShowInstructions;
      end; { case }
    end { then }
    else if UserResponse = Down then begin
      { Scroll one down through options; look if the setting is the lowest one }
      if Option < ExitGame then begin
        Screen.DeleteOptionBox (Option);
        Inc (Option);
        ShowCurrentBox;
      end; { then }
    end { else }
    else if UserResponse = Up then begin
      { Scroll one up through options; look if the option is the highest one }
      if Option > PlayGame then begin
        Screen.DeleteOptionBox (Option);
        Dec (Option);
        ShowCurrentBox;
      end { then }
    end { else then }
    else if UserResponse = Left then begin
      { Chosen to modify the current option -> look if possible }
      case Option of
        ChangeFullRows : begin
                           { Decrease amount of full rows }
                           if FullRows > 0 then begin
                             Dec (FullRows);
                           end; { then }
                         end;
        ChangePlayMode : begin
                           { Decrease play mode chosen }
                           if PlayMode > Small then begin
                             Dec (PlayMode);
                             Settings := StdSettings [PlayMode];
                             Field.ClearField;
                             Screen.ClearScreen;
                             Screen.DrawBorder;
                             Screen.DrawField;
                             WriteMainMenu;
                             ShowSettings;
                             ShowCurrentBox;
                           end; { then }
                         end;
        SoundOnOff     : begin
                           { Sound has to be turned off }
                           SoundOn := false;
                         end;
      end; { case }
      ShowSettings;
    end { else then }
    else if UserResponse = Right then begin
      { Chosen to modify the current option -> look if possible }
      case Option of
        ChangeFullRows : begin
                           { Increase amount of full rows }
                           if FullRows < StdSettings [PlayMode, FieldLength] -
                              StdSettings [PlayMode, BlockLength] then begin
                             Inc (FullRows);
                           end; { then }
                         end;
        ChangePlayMode : begin
                           { Increase play mode chosen }
                           if PlayMode < Large then begin
                             Inc (PlayMode);
                             Settings := StdSettings [PlayMode];
                             Field.ClearField;
                             Screen.ClearScreen;
                             Screen.DrawBorder;
                             Screen.DrawField;
                             WriteMainMenu;
                             ShowSettings;
                             ShowCurrentBox;
                           end; { then }
                         end;
        SoundOnOff     : begin
                           { Sound has to be turned on }
                           SoundOn := true;
                         end;
      end; { case }
      ShowSettings;
    end { else then }
    else if UserResponse = Esc then begin
      Finished := true;
    end; { else then }

  until Finished;

  Screen.ReturnTextMode;
end. { ChiFu }